const gfx = @import("../gfx.zig");

pub const Color = @Vector(4, f32);

pub fn EvenOrderMatrix(comptime n: usize, comptime Element: type) type {
	return [n]@Vector(n, Element);
}

pub const Transform2D = extern struct {
	xbasis: Vector = .{1, 0},
	ybasis: Vector = .{0, 1},
	origin: Vector = @splat(0),

	const Vector = @Vector(2, f32);
};

pub const ProjectionMatrix = EvenOrderMatrix(4, f32);

pub fn orthographic_projection(near: f32, far: f32, viewport: gfx.Rect) EvenOrderMatrix(4, f32) {
	const width = viewport.right - viewport.left;
	const height = viewport.bottom - viewport.top;

	return .{
		.{2 / width, 0, 0, 0},
		.{0, 2 / height, 0, 0},
		.{0, 0, 1 / (far - near), 0},
		.{-((viewport.left + viewport.right) / width), -((viewport.top + viewport.bottom) / height), near / (near - far), 1},
	};
}
